@@ -40,3 +40,9 @@ data("ADBR70")
 ## ------------------------------------------------------------------------
 (chmort(ADBR70,Period = 120))
 
+## ------------------------------------------------------------------------
+(chmort(ADBR70,Period = 120, PeriodEnd = "2013-09"))
+
+## ------------------------------------------------------------------------
+(chmortp(ADBR70))
+
